package cn.com.ctop.kuaishou.modules.ai.controller;

import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.ai.entity.KuaishouPopulationPackage;
import cn.com.ctop.kuaishou.modules.ai.service.IKuaishouPopulationPackageService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 项目绑定的人群包模板
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-05-06
 */
@Slf4j
@Api(tags = "项目绑定的人群包模板")
@RestController
@RequestMapping("/kuaishouPopulationPackage")
public class KuaishouPopulationPackageController {
    @Autowired
    private IKuaishouPopulationPackageService kuaishouPopulationPackageService;

    /**
     * 分页列表查询
     *
     * @param kuaishouPopulationPackage
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @ApiOperation(value = "项目绑定的人群包模板-分页列表查询", notes = "项目绑定的人群包模板-分页列表查询")
    @GetMapping(value = "/list")
    public Result<Object> queryPageList(KuaishouPopulationPackage kuaishouPopulationPackage,
                                        @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                        @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            return kuaishouPopulationPackageService.queryPageList(kuaishouPopulationPackage, pageNo, pageSize);
        } catch (Exception e) {
            log.error("查询人群包列表异常", e);
        }
        return Result.error("查询人群包列表失败");
    }

    /**
     * 查询推送结果
     *
     * @param
     * @return
     */
    @PostMapping(value = "/queryPushResults")
    public Result<Object> queryPushResults(@RequestBody JSONObject requestJson) {
        try {
            Long id = requestJson.getLong("id");
            JSONArray accountIds = requestJson.getJSONArray("accountIds");
            if (Check.isNull(accountIds) || Check.isNull(id)) {
                return Result.error("缺少参数");
            }
            Integer pageNo = requestJson.getInteger("pageNo");
            Integer pageSize = requestJson.getInteger("pageSize");
            if (Check.isNull(pageNo)) {
                pageNo = 1;
            }
            if (Check.isNull(pageSize)) {
                pageSize = 10;
            }
            PageHelper.startPage(pageNo, pageSize);
            return kuaishouPopulationPackageService.queryPushResults(id, accountIds);
        } catch (Exception e) {
            log.error("查询推送结果异常", e);
        }
        return Result.error("查询失败");
    }

    /**
     * 查询推送历史
     *
     * @param
     * @return
     */
    @GetMapping(value = "/queryPushHistory")
    public Result<Object> queryPushHistory(Long id, @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo, @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            if (Check.isNull(id)) {
                return Result.error("缺少参数");
            }
            PageHelper.startPage(pageNo, pageSize);
            return kuaishouPopulationPackageService.queryPushHistory(id);
        } catch (Exception e) {
            log.error("查询异常", e);
        }
        return Result.error("查询失败");
    }

    /**
     * 查询相关账户
     *
     * @param
     * @return org.jeecg.common.api.vo.Result<java.lang.Object>
     * @throws
     * @author ZHAOXA
     */
    @GetMapping(value = "/getRelatedAccounts")
    public Result<Object> getRelatedAccounts(Long id, @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo, @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            if (Check.isNull(id)) {
                return Result.error("缺少参数");
            }
            PageHelper.startPage(pageNo, pageSize);
            return kuaishouPopulationPackageService.getRelatedAccounts(id);
        } catch (Exception e) {
            log.error("查询异常", e);
        }
        return Result.error("查询失败");

    }

    /**
     * 查询相关组
     *
     * @param
     * @return org.jeecg.common.api.vo.Result<java.lang.Object>
     * @throws
     * @author ZHAOXA
     */
    @GetMapping(value = "/getRelatedGroups")
    public Result<Object> getRelatedGroups(Long id,Long accountId, @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo, @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            if (Check.isNull(id)) {
                return Result.error("缺少参数");
            }
            PageHelper.startPage(pageNo, pageSize);
            return kuaishouPopulationPackageService.getRelatedGroups(id,accountId);
        } catch (Exception e) {
            log.error("查询异常", e);
        }
        return Result.error("查询失败");
    }
    /**
     * 查询项目下有效账户
     *
     * @param projectId 账户ID
     * @return org.jeecg.common.api.vo.Result<java.lang.Object>
     * @throws
     * @author ZHAOXA
     */
    @GetMapping(value = "/getAccountByProjectId")
    public Result<Object> getAccountByProjectId(Long projectId, String id,
                                                @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            if (Check.isNull(projectId)) {
                return Result.error("请选择项目");
            }
            if (Check.isNull(id)) {
                return Result.error("请选择应用");
            }
            PageHelper.startPage(pageNo, pageSize);
            return kuaishouPopulationPackageService.getAccountByProjectId(projectId, id);
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    @RequestMapping(value = "/crowdListQuery")
    public Result<Object> crowdListQuery() {
        try {
            kuaishouPopulationPackageService.crowdListQuery();
            return Result.ok("success");
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    @RequestMapping(value = "/checkPopulationPackage")
    public Result<Object> checkPopulationPackage(String signature) {
        try {
           return kuaishouPopulationPackageService.checkPopulationPackage(signature);
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }
    /**
     * 添加
     *
     * @param kuaishouPopulationPackage
     * @return
     */
    @ApiOperation(value = "项目绑定的人群包模板-添加", notes = "项目绑定的人群包模板-添加")
    @PostMapping(value = "/add")
    public Result<Object> add(@RequestBody KuaishouPopulationPackage kuaishouPopulationPackage) {
        Result<KuaishouPopulationPackage> result = new Result<>();
        try {
            if (Check.isNull(kuaishouPopulationPackage.getType())) {
                return Result.error("请选择匹配类型");
            }
            if (Check.isNull(kuaishouPopulationPackage.getOrientationName())) {
                return Result.error("请填写人群包名称");
            }
            if (Check.isNull(kuaishouPopulationPackage.getUrl())) {
                return Result.error("请填写文件路径");
            }
            if (Check.isNull(kuaishouPopulationPackage.getProjectId())) {
                return Result.error("请选择项目");
            }
            return kuaishouPopulationPackageService.createEntity(kuaishouPopulationPackage);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("上传异常");
        }
        return Result.error("上传失败");
    }

    /**
     * 编辑
     *
     * @param kuaishouPopulationPackage
     * @return
     */
    @ApiOperation(value = "项目绑定的人群包模板-编辑", notes = "项目绑定的人群包模板-编辑")
    @PostMapping(value = "/push")
    public Result<Object> push(@RequestBody JSONObject request) {
        try {
            JSONArray accountIds = request.getJSONArray("accountIds");
            Long id = request.getLong("id");
            if (Check.isNull(accountIds) || Check.isNull(id)) {
                return Result.error("缺少参数");
            }
            return kuaishouPopulationPackageService.push(accountIds, id);
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("推送失败");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "项目绑定的人群包模板-通过id删除", notes = "项目绑定的人群包模板-通过id删除")
    @GetMapping(value = "/delete")
    public Result<Object> delete(Long id) {
        try {
            if (Check.isNull(id)) {
                return Result.error("参数为空");
            }
            KuaishouPopulationPackage populationPackage = new KuaishouPopulationPackage();
            populationPackage.setId(id);
            populationPackage.setDataStatus(0);//逻辑删除
            boolean b = kuaishouPopulationPackageService.updateById(populationPackage);
            if (b) {
                return Result.ok("删除成功!");
            }
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
        }
        return Result.error("删除失败!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "项目绑定的人群包模板-批量删除", notes = "项目绑定的人群包模板-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouPopulationPackage> deleteBatch(@RequestParam(name = "ids") String ids) {
        Result<KuaishouPopulationPackage> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouPopulationPackageService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "项目绑定的人群包模板-通过id查询", notes = "项目绑定的人群包模板-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouPopulationPackage> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouPopulationPackage> result = new Result<>();
        KuaishouPopulationPackage kuaishouPopulationPackage = kuaishouPopulationPackageService.getById(id);
        if (kuaishouPopulationPackage == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaishouPopulationPackage);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaishouPopulationPackage> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaishouPopulationPackage kuaishouPopulationPackage = JSON.parseObject(deString, KuaishouPopulationPackage.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaishouPopulationPackage, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaishouPopulationPackage> pageList = kuaishouPopulationPackageService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "项目绑定的人群包模板列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaishouPopulationPackage.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("项目绑定的人群包模板列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaishouPopulationPackage> listKuaishouPopulationPackages = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouPopulationPackage.class, params);
                kuaishouPopulationPackageService.saveBatch(listKuaishouPopulationPackages);
                return Result.ok("文件导入成功！数据行数:" + listKuaishouPopulationPackages.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
